from enum import Enum


class ScorerName(str, Enum):
    VALUE_0 = "_completeness_gpt"
    VALUE_1 = "_context_adherence_luna"
    VALUE_10 = "_input_pii"
    VALUE_11 = "_sexist"
    VALUE_12 = "_input_sexist"
    VALUE_13 = "_sexist_gpt"
    VALUE_14 = "_input_sexist_gpt"
    VALUE_15 = "_tone"
    VALUE_16 = "_input_tone"
    VALUE_17 = "_toxicity"
    VALUE_18 = "_toxicity_gpt"
    VALUE_19 = "_input_toxicity"
    VALUE_2 = "_context_relevance"
    VALUE_20 = "_input_toxicity_gpt"
    VALUE_21 = "_user_registered"
    VALUE_22 = "_user_submitted"
    VALUE_23 = "_user_generated"
    VALUE_24 = "_user_finetuned"
    VALUE_25 = "_uncertainty"
    VALUE_26 = "_bleu"
    VALUE_27 = "_cost"
    VALUE_28 = "_rouge"
    VALUE_29 = "_prompt_injection_gpt"
    VALUE_3 = "_chunk_attribution_utilization_gpt"
    VALUE_30 = "_prompt_injection"
    VALUE_31 = "_rag_nli"
    VALUE_32 = "_adherence_nli"
    VALUE_33 = "_completeness_nli"
    VALUE_34 = "_chunk_attribution_utilization_nli"
    VALUE_35 = "_instruction_adherence"
    VALUE_36 = "_ground_truth_adherence"
    VALUE_37 = "_tool_selection_quality"
    VALUE_38 = "_tool_selection_quality_luna"
    VALUE_39 = "_tool_error_rate"
    VALUE_4 = "_factuality"
    VALUE_40 = "_tool_error_rate_luna"
    VALUE_41 = "_action_completion_luna"
    VALUE_42 = "_agentic_session_success"
    VALUE_43 = "_action_advancement_luna"
    VALUE_44 = "_agentic_workflow_success"
    VALUE_45 = "_generic_wizard"
    VALUE_46 = "_customized_completeness_gpt"
    VALUE_47 = "_customized_factuality"
    VALUE_48 = "_customized_groundedness"
    VALUE_49 = "_customized_chunk_attribution_utilization_gpt"
    VALUE_5 = "_groundedness"
    VALUE_50 = "_customized_instruction_adherence"
    VALUE_51 = "_customized_ground_truth_adherence"
    VALUE_52 = "_customized_prompt_injection_gpt"
    VALUE_53 = "_customized_tool_selection_quality"
    VALUE_54 = "_customized_tool_error_rate"
    VALUE_55 = "_customized_agentic_session_success"
    VALUE_56 = "_customized_agentic_workflow_success"
    VALUE_57 = "_customized_sexist_gpt"
    VALUE_58 = "_customized_input_sexist_gpt"
    VALUE_59 = "_customized_toxicity_gpt"
    VALUE_6 = "_latency"
    VALUE_60 = "_customized_input_toxicity_gpt"
    VALUE_7 = "_prompt_perplexity"
    VALUE_8 = "_protect_status"
    VALUE_9 = "_pii"

    def __str__(self) -> str:
        return str(self.value)
